import * as moment from 'moment';

const pad = (number: number): string => number.toString().padStart(2, '0');

class DateTime {
  private readonly utcYear: number;
  private readonly utcMonth: number;
  private readonly utcDay: number;
  private readonly utcHours: number;
  private readonly utcMinutes: number;
  private readonly year: number;
  private readonly month: number;
  private readonly day: number;
  private readonly hours: number;
  private readonly minutes: number;

  private constructor(isoDate: string) {
    const date = new Date(isoDate);

    this.utcYear = date.getUTCFullYear();
    this.utcMonth = date.getUTCMonth() + 1;
    this.utcDay = date.getUTCDate();
    this.utcHours = date.getUTCHours();
    this.utcMinutes = date.getUTCMinutes();

    this.year = date.getFullYear();
    this.month = date.getMonth() + 1;
    this.day = date.getDate();
    this.hours = date.getHours();
    this.minutes = date.getMinutes();
  }
  static of(isoDate: string): DateTime {
    return new DateTime(isoDate);
  }

  getUtcMonth(): number {
    return this.utcMonth;
  }

  getUtcDay(): number {
    return this.utcDay;
  }

  getUtcHours(): number {
    return this.utcHours;
  }

  getUtcMinutes(): number {
    return this.utcMinutes;
  }

  getUtcYear(): number {
    return this.utcYear;
  }

  toHuman(): string {
    return `${pad(this.day)}/${pad(this.month)}/${this.year} ${pad(this.hours)}:${pad(this.minutes)}`;
  }
}

const ISO_DATE = '2020-10-06T19:10:00Z';

describe('DateTime', () => {
  it('Should have an utc year', () => {
    expect(DateTime.of(ISO_DATE).getUtcYear()).toBe(2020);
  });

  it('Should have an utc month', () => {
    expect(DateTime.of(ISO_DATE).getUtcMonth()).toBe(10);
  });

  it('Should have an utc day', () => {
    expect(DateTime.of(ISO_DATE).getUtcDay()).toBe(6);
  });

  it('Should have an utc hour', () => {
    expect(DateTime.of(ISO_DATE).getUtcHours()).toBe(19);
  });

  it('Should have an utc minutes', () => {
    expect(DateTime.of(ISO_DATE).getUtcMinutes()).toBe(10);
  });

  it('Should display in human format', () => {
    expect(DateTime.of(ISO_DATE).toHuman()).toBe(moment(ISO_DATE).format('DD/MM/YYYY HH:mm'));
  });
});
