import moment = require('moment');

const DATE_PATTERN = /^\d{4}-\d{2}-\d{2}$/;

const padTwoCharacters = (digit: number): string => digit.toString().padStart(2, '0');

const assertDateFormat = (isoDate: string): void => {
  if (!DATE_PATTERN.test(isoDate)) {
    throw new Error('The date is malformed.');
  }
};

const assertValidDate = (isoDate: string): void => {
  if (!moment(isoDate, 'YYYY-MM-DD', true).isValid()) {
    throw new Error("The date doesn't exist.");
  }
};

export class LocalDate {
  private month: number;
  private year: number;
  private day: number;

  constructor(isoDate: string) {
    assertDateFormat(isoDate);
    assertValidDate(isoDate);

    [this.year, this.month, this.day] = isoDate.split('-').map((member) => parseInt(member, 10));
  }

  public getMonth(): number {
    return this.month;
  }

  public getYear(): number {
    return this.year;
  }

  public getDay(): number {
    return this.day;
  }

  public toHumanFormat(): string {
    return `${padTwoCharacters(this.day)}/${padTwoCharacters(this.month)}/${this.year}`;
  }
}
