import axios, { AxiosInstance } from 'axios';

const SERVER_URL = 'http://localhost:8080';

const enhanceErrorMessage = (error: any) => {
  error.message += ` → ${error.response.data.errorType} ("${error.response.data.message}")`;
  return Promise.reject(error);
};

export const createAnonymousAxios = () =>
  axios.create({
    baseURL: SERVER_URL,
  });

const createUserAxios = (token: string): AxiosInstance => {
  const instance = axios.create({
    ...{
      baseURL: SERVER_URL,
    },
    headers: {
      Authorization: `Bearer ${token}`,
    },
  });
  instance.interceptors.response.use((response) => response, enhanceErrorMessage);
  return instance;
};

export const createGivenUserAxios = async (username: string, password: string): Promise<AxiosInstance> => {
  const axiosInstance = createAnonymousAxios();
  const { data: { 'id_token': token } } = await axiosInstance.post('/api/authenticate', {username, password});

  return createUserAxios(token);
};

export const createAdminAxiosInstance = () => createGivenUserAxios('admin', 'admin');
export const createUserAxiosInstance = () => createGivenUserAxios('user', 'user');
