/**
 *
 */
package com.ippon.borestop.common.infrastructure.primary;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
import com.ippon.borestop.common.infrastructure.primary.BorestopFieldError.BorestopFieldErrorBuilder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

@JsonDeserialize(builder = BorestopFieldErrorBuilder.class)
@ApiModel(description = "Error for a field validation")
public class BorestopFieldError {

  @ApiModelProperty(value = "Path to the field in error", example = "address.country", required = true)
  private final String fieldPath;

  @ApiModelProperty(value = "Technical reason for the invalidation", example = "user.mandatory", required = true)
  private final String reason;

  @ApiModelProperty(value = "Human readable message for the invalidation", example = "Le champ doit être renseigné", required = true)
  private final String message;

  private BorestopFieldError(BorestopFieldErrorBuilder builder) {
    fieldPath = builder.fieldPath;
    reason = builder.reason;
    message = builder.message;
  }

  public static BorestopFieldErrorBuilder builder() {
    return new BorestopFieldErrorBuilder();
  }

  public String getFieldPath() {
    return fieldPath;
  }

  public String getReason() {
    return reason;
  }

  public String getMessage() {
    return message;
  }

  @JsonPOJOBuilder(withPrefix = "")
  public static class BorestopFieldErrorBuilder {

    private String fieldPath;
    private String reason;
    private String message;

    public BorestopFieldErrorBuilder fieldPath(String fieldPath) {
      this.fieldPath = fieldPath;

      return this;
    }

    public BorestopFieldErrorBuilder reason(String reason) {
      this.reason = reason;

      return this;
    }

    public BorestopFieldErrorBuilder message(String message) {
      this.message = message;

      return this;
    }

    public BorestopFieldError build() {
      return new BorestopFieldError(this);
    }
  }
}
