package com.ippon.borestop.activity.infrastructure.secondary;

import static com.ippon.borestop.activity.domain.Category.*;
import static org.assertj.core.api.Assertions.*;

import com.ippon.borestop.activity.domain.Idea;
import com.ippon.borestop.common.BorestopIntTest;
import java.time.Duration;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.boot.web.server.LocalServerPort;

@BorestopIntTest
class BoredApiIdeasRepositoryCommunicationIntTest {

  @LocalServerPort
  private int port;

  @Autowired
  private FakeBoredApi fake;

  @Autowired
  private RestTemplateBuilder restBuilder;

  private BoredApiIdeasRepository repository;

  @BeforeEach
  void loadRepository() {
    fake.reset();

    repository = new BoredApiIdeasRepository(restBuilder, properties());
  }

  private BoredApiProperties properties() {
    BoredApiProperties properties = new BoredApiProperties();

    properties.setUrl("http://localhost:" + port + "/fake-bored-api");
    properties.setConnectTimeout(Duration.ofMillis(10));
    properties.setReadTimeout(Duration.ofMillis(50));

    return properties;
  }

  @Test
  void shouldGracefullyHandleTimeouts() {
    fake.slowDown();

    Idea idea = repository.next();

    assertThat(idea.getLabel()).isEqualTo("Go grab a beer");
    assertThat(idea.getCategory()).isEqualTo(RELAXATION);
  }
}
