package fr.ippon.life;

import java.util.Set;

public record Cell(int row, int column) {
  private static final int LOW_POPULATION_THRESHOLD = 2;
  private static final int HIGH_POPULATION_THRESHOLD = 3;
  private static final int BORN_THRESHOLD = HIGH_POPULATION_THRESHOLD;

  boolean stayAlive(Set<Cell> aliveCells) {
    long neighbourgCount = countNeigbours(aliveCells);

    return neighbourgCount == LOW_POPULATION_THRESHOLD || neighbourgCount == HIGH_POPULATION_THRESHOLD;
  }

  boolean born(Set<Cell> aliveCells) {
    return countNeigbours(aliveCells) == BORN_THRESHOLD;
  }

  private long countNeigbours(Set<Cell> aliveCells) {
    return aliveCells.stream().filter(this::isNeighbour).count();
  }

  private boolean isNeighbour(Cell other) {
    if (other.equals(this)) {
      return false;
    }

    return delta(row, other.row) <= 1 && delta(column, other.column) <= 1;
  }

  private static int delta(int first, int second) {
    return Math.abs(first - second);
  }
}
