package fr.ippon.life;

import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Generation {

  private Set<Cell> aliveCells;

  public Generation(Cell... aliveCells) {
    this(Set.of(aliveCells));
  }

  private Generation(Set<Cell> cells) {
    aliveCells = cells;
  }

  public Generation next() {
    Stream<Cell> stayAlive = aliveCells.stream().filter(cell -> cell.stayAlive(aliveCells));

    Stream<Cell> born = buildGrid().deadCells().stream().filter(cell -> cell.born(aliveCells));

    Set<Cell> cells = Stream.concat(stayAlive, born).collect(Collectors.toUnmodifiableSet());

    return new Generation(cells);
  }

  public Collection<Cell> getAliveCells() {
    return aliveCells;
  }

  private Grid buildGrid() {
    return new Grid(aliveCells);
  }
}
