package fr.ippon.life;

import java.util.function.IntFunction;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class StringRenderer implements Renderer<String> {

  @Override
  public String run(Grid grid) {
    return new GridRenderer(grid).run();
  }

  private static class GridRenderer {

    private final Grid grid;

    private GridRenderer(Grid grid) {
      this.grid = grid;
    }

    private String run() {
      return IntStream
        .range(grid.topRow(), grid.bottomRow())
        .mapToObj(Integer::valueOf)
        .map(row -> rowRepresentation(grid, row))
        .collect(Collectors.joining(System.lineSeparator(), "", System.lineSeparator()));
    }

    private String rowRepresentation(Grid grid, Integer row) {
      return IntStream.range(grid.leftColumn(), grid.rightColumn()).mapToObj(cellRepresentation(row)).collect(Collectors.joining());
    }

    private IntFunction<String> cellRepresentation(Integer row) {
      return column -> {
        if (grid.isAlive(new Cell(row, column))) {
          return "o";
        }

        return "x";
      };
    }
  }
}
