package fr.ippon.life;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

class GameOfLifeTest {

  @Test
  void nextGenerationShouldHaveAllDeadCellsForAllDeadGrid() {
    Generation nextGeneration = new Generation().next();

    assertThat(nextGeneration.getAliveCells()).isEmpty();
  }

  @Test
  void nextGenerationShouldHaveAllDeadCellsFromOneAliveCell() {
    Generation nextGeneration = new Generation(cell(1, 1)).next();

    assertThat(nextGeneration.getAliveCells()).isEmpty();
  }

  @Test
  void nextGenerationShouldHaveAllDeadCellsFromTwoContiguousAliveCells() {
    Generation nextGeneration = new Generation(cell(1, 1), cell(2, 1)).next();

    assertThat(nextGeneration.getAliveCells()).isEmpty();
  }

  @Test
  void nextGenerationShouldHaveFourAliveCellsFromTriangularConfiguration() {
    Cell first = cell(1, 2);
    Cell second = cell(2, 1);
    Cell third = cell(2, 2);
    Generation nextGeneration = new Generation(first, second, third).next();

    assertThat(nextGeneration.getAliveCells()).containsExactlyInAnyOrder(cell(1, 1), first, second, third);
  }

  @Test
  void nextGenerationShouldHaveOneColumnFromOneLine() {
    Generation nextGeneration = threeCellsColumn().next();

    assertThat(nextGeneration.getAliveCells()).containsExactlyInAnyOrder(cell(5, 4), cell(5, 5), cell(5, 6));
  }

  @Test
  void shouldAdvanceInGenerations() {
    Generation nextGeneration = threeCellsColumn().next().next();

    assertThat(nextGeneration.getAliveCells()).containsExactlyInAnyOrder(cell(4, 5), cell(5, 5), cell(6, 5));
  }

  private Generation threeCellsColumn() {
    return new Generation(cell(4, 5), cell(5, 5), cell(6, 5));
  }

  private static Cell cell(int row, int column) {
    return new Cell(row, column);
  }
}
