package fr.ippon.life;

import static org.assertj.core.api.Assertions.*;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;
import org.junit.jupiter.api.Test;

class StringRendererUnitTest {

  private static final StringRenderer renderer = new StringRenderer();

  @Test
  void shouldDisplayGridWithOneAliveCell() {
    assertThat(renderer.run(grid().alive(1, 1).build())).isEqualTo(join("xxx", "xox", "xxx"));
  }

  @Test
  void shouldDisplayGridWithTwoAliveCell() {
    assertThat(renderer.run(grid().alive(1, 1).alive(1, 3).build())).isEqualTo(join("xxxxx", "xoxox", "xxxxx"));
  }

  @Test
  void shouldDisplayGridWithTwoDistantAliveCell() {
    assertThat(renderer.run(grid().alive(1, 1).alive(1, 7).build())).isEqualTo(join("xxxxxxxxx", "xoxxxxxox", "xxxxxxxxx"));
  }

  private static String join(String... parts) {
    return Arrays.stream(parts).collect(Collectors.joining(System.lineSeparator(), "", System.lineSeparator()));
  }

  private GridBuilder grid() {
    return new GridBuilder();
  }

  private static class GridBuilder {

    private final Set<Cell> cells = new HashSet<>();

    public GridBuilder alive(int row, int column) {
      cells.add(new Cell(row, column));

      return this;
    }

    public Grid build() {
      return new Grid(cells);
    }
  }
}
