package fr.ippon.monad;

import java.util.function.Function;

public class State<S, T> {

  private final Function<S, Result<S, T>> runner;

  private State(Function<S, Result<S, T>> runner) {
    this.runner = runner;
  }

  public static <S, T> State<S, T> pure(T value) {
    return new State<>(state -> new Result<>(state, value));
  }

  public static <S> State<S, Void> set(S value) {
    return new State<>(state -> new Result<>(value, null));
  }

  public static <S> State<S, S> get() {
    return new State<>(state -> new Result<>(state, state));
  }

  public <U> State<S, U> bind(Function<T, State<S, U>> binder) {
    return new State<>(
      initial -> {
        Result<S, T> result = runner.apply(initial);

        return binder.apply(result.result()).run(result.updated());
      }
    );
  }

  public Result<S, T> run(S initial) {
    return runner.apply(initial);
  }

  public static record Result<S, T>(S updated, T result) {}
}
