package fr.ippon.kata.pagination;

import java.util.function.IntFunction;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class Pagination {

  private static final String SPACE = " ";
  private static final String ELLIPSIS = " … ";

  private final String representation;

  private Pagination(PaginationBuilder builder) {
    representation = buildRepresentation(builder);
  }

  private String buildRepresentation(PaginationBuilder builder) {
    if (builder.total > 7 && builder.current > builder.total - 4) {
      return endingEllipsis(builder);
    }

    if (builder.total <= 7) {
      return fullPages(builder);
    }

    if (builder.current < 5) {
      return startingEllipsis(builder);
    }

    return twoEllipsisPagination(builder);
  }

  private String endingEllipsis(PaginationBuilder builder) {
    return "1" + ELLIPSIS + rangeRepresentation(builder.total - 4, builder.total, builder.current);
  }

  private String fullPages(PaginationBuilder builder) {
    int end = builder.total;

    return rangeRepresentation(1, end, builder.current);
  }

  private String startingEllipsis(PaginationBuilder builder) {
    return rangeRepresentation(1, 5, builder.current) + ELLIPSIS + builder.total;
  }

  private String twoEllipsisPagination(PaginationBuilder builder) {
    int start = builder.current - 1;
    int end = builder.current + 1;

    return "1" + ELLIPSIS + rangeRepresentation(start, end, builder.current) + ELLIPSIS + builder.total;
  }

  private String rangeRepresentation(int start, int end, int current) {
    return IntStream.range(start, end + 1).mapToObj(toPage(current)).collect(Collectors.joining(SPACE));
  }

  private IntFunction<String> toPage(int current) {
    return value -> {
      if (current == value) {
        return currentPageRepresentation(value);
      }

      return String.valueOf(value);
    };
  }

  private String currentPageRepresentation(int value) {
    return "(" + value + ")";
  }

  public static PaginationBuilder current(int current) {
    return new PaginationBuilder(current);
  }

  private String display() {
    return representation;
  }

  public static class PaginationBuilder {

    private final int current;
    private int total;

    public PaginationBuilder(int current) {
      this.current = current;
    }

    public PaginationBuilder total(int total) {
      this.total = total;

      return this;
    }

    public String display() {
      return new Pagination(this).display();
    }
  }
}
