package fr.ippon;

import static org.assertj.core.api.Assertions.*;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import org.junit.jupiter.api.Test;

public class PlayingWithThreads {

  private static boolean threadDoneFlag;

  private static AtomicBoolean threadDoneAtomicFlag = new AtomicBoolean(false);

  @Test
  public void playingWithSimpleDateFormat() throws InterruptedException {
    Set<Integer> days = Collections.newSetFromMap(new ConcurrentHashMap<>());
    ExecutorService executor = Executors.newFixedThreadPool(10);

    SimpleDateFormat format = new SimpleDateFormat("dd");
    AtomicInteger counter = new AtomicInteger(1);
    for (int i = 1; i < 32; ++i) {
      executor.submit(
        () -> {
          Date date = new GregorianCalendar(2018, Calendar.JANUARY, counter.getAndIncrement()).getTime();
          days.add(Integer.parseInt(format.format(date)));
        }
      );
    }

    executor.shutdown();
    executor.awaitTermination(3, TimeUnit.SECONDS);

    // assertThat(days).hasSize(31);
    assertThat(days.size()).isLessThan(31);
  }

  @Test
  public void firstAttemptToStopAThread() throws InterruptedException {
    AtomicBoolean threadStopped = new AtomicBoolean();

    new Thread(
      () -> {
        while (!threadDoneFlag) {}

        threadStopped.set(true);
      }
    )
      .start();

    Thread.sleep(500);

    threadDoneFlag = true;

    Thread.sleep(50);
    assertThat(threadStopped.get()).isFalse();
    // assertThat(threadStopped.get()).isTrue();
  }

  @Test
  public void secondAttemptToStopAThread() throws InterruptedException {
    AtomicBoolean threadStopped = new AtomicBoolean();

    new Thread(
      () -> {
        while (!threadDoneFlag) {
          System.out.println(threadDoneFlag); // Ok, let's just log the flag
        }

        threadStopped.set(true);
      }
    )
      .start();

    Thread.sleep(500);

    threadDoneFlag = true;

    Thread.sleep(50);

    // assertThat(threadStopped.get()).isFalse();
    assertThat(threadStopped.get()).isTrue();
  }

  @Test
  public void thirdAttemptToStopAThread() throws InterruptedException {
    AtomicBoolean threadStopped = new AtomicBoolean();

    new Thread(
      () -> {
        while (!threadDoneAtomicFlag.get()) {}

        threadStopped.set(true);
      }
    )
      .start();

    Thread.sleep(500);

    threadDoneAtomicFlag.set(true);

    Thread.sleep(50);

    // assertThat(threadStopped.get()).isFalse();
    assertThat(threadStopped.get()).isTrue();
  }
}
