package fr.ippon.streams;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Mall {

  private final List<Shop> shops;

  public Mall(Shop... shops) {
    this.shops = Arrays.asList(shops);
  }

  public Collection<String> getShopNames() {
    return shops.stream().map(Shop::getName).sorted().collect(Collectors.toList());
  }

  public String getEmployeesMeanAge() {
    return employeesAgeStream()
      .map(BigDecimal::valueOf)
      .reduce(BigDecimal::add)
      .map(this::toAverage)
      .orElse(BigDecimal.ZERO)
      .toPlainString();
  }

  private Stream<Integer> employeesAgeStream() {
    return shops.stream().flatMap(this::toEmployeesAges);
  }

  private Stream<Integer> toEmployeesAges(Shop shop) {
    return toEmployees(shop).map(Employee::getAge);
  }

  private BigDecimal toAverage(BigDecimal sum) {
    return sum.divide(new BigDecimal(employeesAgeStream().count()), 2, RoundingMode.HALF_UP);
  }

  public Collection<String> getEmployeesOver(int age) {
    return shops
      .stream()
      .flatMap(this::toEmployees)
      .filter(employee -> overAge(age, employee))
      .map(Employee::getName)
      .sorted()
      .collect(Collectors.toList());
  }

  private Stream<Employee> toEmployees(Shop shop) {
    return shop.getEmployees().stream();
  }

  private boolean overAge(int age, Employee employee) {
    return employee.getAge() > age;
  }
}
