package fr.ippon.markov;

import static org.assertj.core.api.Assertions.*;

import java.util.Map;

import org.junit.jupiter.api.Test;

class MarkovChainTest {
  private final MarkovChain markov = new MarkovChain();

  @Test
  void shouldGetStatsForOneWord() {
    markov.learn("Les");

    assertThat(markov.stats("Les")
        .get()).isEmpty();
  }

  @Test
  void shoulGetStatsForTwoWords() {
    markov.learn("Les hommes");

    assertThat(markov.stats("Les")
        .get()).containsExactly(stat("hommes", 1f));
  }

  @Test
  void shoulGetStatsWithMultipleFollowers() {
    markov.learn("Les hommes les plus grands");

    assertThat(markov.stats("Les")
        .get()).containsExactly(stat("hommes", 0.5f),
            new Stat("plus", 0.5f));
    assertThat(markov.stats("hommes")
        .get()).containsExactly(stat("les", 1f));
  }

  @Test
  void shoulGetStats() {
    markov.learn("Les hommes les plus grands");

    assertThatStats(markov.stats()).hasWord("les")
        .withStat("hommes", 0.5f)
        .withStat("plus", 0.5f)
        .and()
        .hasWord("hommes")
        .withStat("les", 1f);
  }

  private static Stat stat(String word, float stat) {
    return new Stat(word, stat);
  }

  private static StatsAsserter assertThatStats(
      Map<String, WordStats> stats) {
    return new StatsAsserter(stats);
  }

  private static class StatsAsserter {

    private final Map<String, WordStats> stats;

    public StatsAsserter(Map<String, WordStats> stats) {
      this.stats = stats;
    }

    public WordStatAsserter hasWord(String word) {
      assertThat(stats).containsKey(word);

      return new WordStatAsserter(stats.get(word), this);
    }

  }

  private static class WordStatAsserter {

    private final WordStats wordStats;
    private final StatsAsserter source;

    public WordStatAsserter(WordStats wordStats,
        StatsAsserter source) {
      this.wordStats = wordStats;
      this.source = source;
    }

    public WordStatAsserter withStat(String word,
        float percent) {
      float result = wordStats.get()
          .stream()
          .filter(stat -> stat.word()
              .equals(word))
          .findFirst()
          .orElseThrow(AssertionError::new)
          .percentage();

      assertThat(result).isEqualTo(percent);

      return this;
    }

    public StatsAsserter and() {
      return source;
    }
  }
}
