package fr.ippon.rental;

import java.util.ArrayList;
import java.util.List;

public class Customer {

  private final String name;
  private final List<Rental> rentals = new ArrayList<Rental>();

  public Customer(String name) {
    this.name = name;
  }

  public void addRental(Rental arg) {
    rentals.add(arg);
  }

  public String getName() {
    return name;
  }

  public String statement(Renderer renderer) {
    StringBuilder result = new StringBuilder(renderer.header(name));

    double totalAmount = 0;
    for (Rental rental : rentals) {
      double amount = calculateAmount(rental);

      result.append(renderer.movie(rental, amount));
      totalAmount += amount;
    }

    result.append(renderer.footer(totalAmount, calculateFrequentRenterPoints()));

    return result.toString();
  }

  private int calculateFrequentRenterPoints() {
    return rentals.stream().mapToInt(Rental::getRenterPoints).sum();
  }

  private double calculateAmount(Rental rental) {
    return rental.amount();
  }
}
