package fr.craft.kata.domain;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public record Firstname(String firstname)
    implements Comparable<Firstname> {
  private static final Pattern FIRSTNAME_ELEMENT = Pattern
      .compile("([a-zA-Z]+)([^a-zA-Z]*)");

  public Firstname(String firstname) {
    assertFirstname(firstname);

    this.firstname = buildFirstname(firstname);
  }

  private String buildFirstname(String firstname) {
    Matcher matcher = FIRSTNAME_ELEMENT.matcher(firstname);

    StringBuilder result = new StringBuilder();
    while (matcher.find()) {
      String name = matcher.group(1);

      result.append(name.substring(0, 1)
          .toUpperCase());

      result.append(name.substring(1, name.length())
          .toLowerCase());

      result.append(matcher.group(2));
    }

    return result.toString();
  }

  private void assertFirstname(String firstname) {
    if (firstname == null || firstname.isBlank()) {
      throw new IllegalArgumentException(
          "You must set a firstname");
    }
  }

  public String get() {
    return firstname();
  }

  @Override
  public int compareTo(Firstname other) {
    return firstname().compareTo(other.firstname());
  }
}
