const FIRST_THRESHOLD = 4;
const LAST_THRESHOLD = 3;
const FIRST_PAGE = 1;

const SEVEN = 7;
const ELLIPSIS = '…';

const pageRepresentation = (currentPage: number) => (page: number): string => {
  if (page === currentPage) {
    return `(${page})`;
  }
  return page.toString();
};
const paginationBetween = (currentPage: number, from: number, to: number): string => {
  const pages: number = to - from + 1;
  return Array(pages)
    .fill(0)
    .map((_, index) => index + from)
    .map(pageRepresentation(currentPage))
    .join(' ');
};

const paginationUntilSeven = (currentPage: number, pages: number): string => paginationBetween(currentPage, FIRST_PAGE, pages);

const firstPartPagination = (currentPage: number, pages: number) =>
  `${paginationBetween(currentPage, FIRST_PAGE, FIRST_THRESHOLD + 1)} ${ELLIPSIS} ${pages}`;

const lastPartPagination = (currentPage: number, pages: number) =>
  `${FIRST_PAGE} ${ELLIPSIS} ${paginationBetween(currentPage, pages - LAST_THRESHOLD - 1, pages)}`;

const paginationOverSeven = (currentPage: number, pages: number): string => {
  if (currentPage <= FIRST_THRESHOLD) {
    return firstPartPagination(currentPage, pages);
  }

  if (currentPage >= pages - LAST_THRESHOLD) {
    return lastPartPagination(currentPage, pages);
  }

  return `${FIRST_PAGE} ${ELLIPSIS} ${currentPage - 1} (${currentPage}) ${currentPage + 1} ${ELLIPSIS} ${pages}`;
};
const paginationSeven = (currentPage: number, pages: number): string => {
  if (pages <= SEVEN) {
    return paginationUntilSeven(currentPage, pages);
  }

  return paginationOverSeven(currentPage, pages);
};

describe('Pagination seven', () => {
  it('Should show (1) when current page is 1 on 1 page', () => {
    expect(paginationSeven(1, 1)).toBe('(1)');
  });

  it('Should show "1 2 (3) 4 5 6 7" when current page is 3 on 7 pages', () => {
    expect(paginationSeven(3, 7)).toBe('1 2 (3) 4 5 6 7');
  });

  it('Should show "1 … 4 (5) 6 … 9" when current page is 5 on 9 pages', () => {
    expect(paginationSeven(5, 9)).toBe('1 … 4 (5) 6 … 9');
  });

  it('Should show "1 2 3 (4) 5 … 9" when current page is 4 on 9 pages', () => {
    expect(paginationSeven(4, 9)).toBe('1 2 3 (4) 5 … 9');
  });

  it('Should show "1 … 5 (6) 7 8 9" when current page is 6 on 9 pages', () => {
    expect(paginationSeven(6, 9)).toBe('1 … 5 (6) 7 8 9');
  });

  it('Should show "1 … 41 (42) 43 … 100" when current page is 42 on 100 pages', () => {
    expect(paginationSeven(42, 100)).toBe('1 … 41 (42) 43 … 100');
  });
});
