package fr.ippon.stringcalculator;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

public class CalculatorTest {

  @Test
  void shouldNotSumWithNullFirstInput() {
    assertThatThrownBy(() -> Calculator.sum(null, "3")).isExactlyInstanceOf(IllegalArgumentException.class);
  }

  @Test
  void shouldNotSumWithNullSecondInput() {
    assertThatThrownBy(() -> Calculator.sum("3", null)).isExactlyInstanceOf(IllegalArgumentException.class);
  }

  @Test
  void shouldSumEmptyInputs() {
    assertThat(Calculator.sum("", "")).isEqualTo("0");
  }

  @Test
  void shouldSumIntegerWithEmptyFirstInput() {
    assertThat(Calculator.sum("", "4")).isEqualTo("4");
  }

  @Test
  void shouldSumIntegerWithEmptySecondInput() {
    assertThat(Calculator.sum("4", "")).isEqualTo("4");
  }

  @Test
  void shouldSumIntegers() {
    assertThat(Calculator.sum("1", "2")).isEqualTo("3");
  }

  @Test
  void shouldSumFloats() {
    assertThat(Calculator.sum("1.01", "2.02")).isEqualTo("3.03");
  }

  @Test
  void shouldSumCommaSeparatedFloats() {
    assertThat(Calculator.sum("1,01", "2,02")).isEqualTo("3.03");
  }

  @Test
  void shouldSumNegativeCommaSeparatedFloats() {
    assertThat(Calculator.sum("-1,01", "-2,02")).isEqualTo("-3.03");
  }

  @Test
  void shouldNotSubstractNullInputs() {
    assertThatThrownBy(() -> Calculator.substract(null, "3")).isExactlyInstanceOf(IllegalArgumentException.class);
  }

  @Test
  void shouldNotSubstractWithNullSecondInputs() {
    assertThatThrownBy(() -> Calculator.substract("3", null)).isExactlyInstanceOf(IllegalArgumentException.class);
  }

  @Test
  void shouldSubstractCommaSeparatedFloats() {
    assertThat(Calculator.substract("1,01", "2,02")).isEqualTo("-1.01");
  }

  @Test
  void shouldSumTooBigIntegers() {
    assertThat(Calculator.sum(String.valueOf(Integer.MAX_VALUE), "2,02")).isEqualTo("2147483649.02");
  }
}
