package fr.ippon.tennis;

import java.util.Arrays;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public enum Score {
  LOVE(0, "0"), //
  FIFTEEN(1, "15"), //
  THIRTY(2, "30"), //
  FOURTY(3, "40"), //
  ENDING(4, "GAME");

  private static final Map<Integer, Score> SCORES = buildScores();

  private final int index;
  private final String display;

  Score(int index, String display) {
    this.index = index;
    this.display = display;
  }

  private static Map<Integer, Score> buildScores() {
    return Arrays.stream(values()).collect(Collectors.toMap(Score::index, Function.identity()));
  }

  private int index() {
    return index;
  }

  Score scores() {
    return Score.of(index + 1);
  }

  private static Score of(int index) {
    return SCORES.getOrDefault(index, ENDING);
  }

  boolean hasScored() {
    return this != LOVE;
  }

  String display() {
    return display;
  }
}
