package fr.ippon.tennis;

import static org.assertj.core.api.Assertions.*;

import java.util.stream.IntStream;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

class GameTest {

  private final Game game = new Game();

  @Test
  void shouldStartAtLoveAll() {
    assertThat(game.score()).isEqualTo("LOVE - ALL");
  }

  @Test
  void playerOneScoresFirstPoint() {
    game.playerOneScores();

    assertThat(game.score()).isEqualTo("15 - 0");
  }

  @Test
  void playerOneScoresTwoPoints() {
    playerOneScores(2);

    assertThat(game.score()).isEqualTo("30 - 0");
  }

  @Test
  void playerOneScoresThreePoints() {
    playerOneScores(3);

    assertThat(game.score()).isEqualTo("40 - 0");
  }

  @Test
  void playerTwoScoreFirstPoint() {
    game.playerTwoScores();

    assertThat(game.score()).isEqualTo("0 - 15");
  }

  @Test
  void playerOneAndTwoScoresFirstPoint() {
    game.playerOneScores();
    game.playerTwoScores();

    assertThat(game.score()).isEqualTo("15 - ALL");
  }

  @Test
  void shouldHaveDeuce() {
    deuce();

    assertThat(game.score()).isEqualTo("DEUCE");
  }

  @Test
  void playerOneShouldHaveAdvantage() {
    deuce();
    game.playerOneScores();

    assertThat(game.score()).isEqualTo("ADVANTAGE PLAYER 1");
  }

  @Test
  void playerTwoShouldHaveAdvantage() {
    deuce();
    game.playerTwoScores();

    assertThat(game.score()).isEqualTo("ADVANTAGE PLAYER 2");
  }

  @Test
  void playerTwoShouldComeBackToDeuce() {
    deuce();
    game.playerOneScores();
    game.playerTwoScores();

    assertThat(game.score()).isEqualTo("DEUCE");
  }

  @Test
  void playerOneShouldComeBackToDeuce() {
    deuce();
    game.playerTwoScores();
    game.playerOneScores();

    assertThat(game.score()).isEqualTo("DEUCE");
  }

  @Test
  void playerOneShouldWinGame() {
    playerOneScores(4);

    assertThat(game.score()).isEqualTo("GAME PLAYER 1");
  }

  @Test
  void playerTwoShouldWinGame() {
    playerTwoScores(4);

    assertThat(game.score()).isEqualTo("GAME PLAYER 2");
  }

  @Test
  @Disabled("This is the test proving that we where wrong from the beginning!")
  void playerOneShouldWinAfterAdvantage() {
    deuce();
    game.playerOneScores();
    game.playerOneScores();

    assertThat(game.score()).isEqualTo("GAME PLAYER 1");
  }

  private void deuce() {
    playerOneScores(3);
    playerTwoScores(3);
  }

  private void playerOneScores(int times) {
    IntStream.range(0, times).forEach(d -> game.playerOneScores());
  }

  private void playerTwoScores(int times) {
    IntStream.range(0, times).forEach(d -> game.playerTwoScores());
  }
}
