package fr.ippon.craft;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

class TryTest {

  @Test
  void shouldCreateSuccessTry() {
    assertThat(Try.of(() -> "Yeah").get()).isEqualTo("Yeah");
  }

  @Test
  void shouldCreateFaillingTry() {
    assertThatThrownBy(() -> Try.of(failing()).get())
      .isExactlyInstanceOf(ExecutionException.class)
      .hasCauseExactlyInstanceOf(RuntimeException.class);
  }

  @Test
  void shouldGetGoodResultForSuccess() {
    assertThat(Try.of(() -> "Yeah").orElse("Hey")).isEqualTo("Yeah");
  }

  @Test
  void shouldGetAlternativeResultWhenFailing() {
    assertThat(Try.of(failing()).orElse("Hey")).isEqualTo("Hey");
  }

  @Test
  void shouldMapSuccessfulResult() {
    assertThat(Try.of(sucess()).map(Result::new).get()).isEqualTo(new Result("Hello"));
  }

  @Test
  void shouldNotMapFailingResult() {
    Try<Result> tryable = Try.of(failing()).map(Result::new);

    assertThatThrownBy(tryable::get).isExactlyInstanceOf(ExecutionException.class).hasCauseExactlyInstanceOf(RuntimeException.class);
  }

  @Test
  void shouldFlatMapSuccessfulTry() {
    assertThat(Try.of(sucess()).flatMap(value -> Try.of(() -> value + " You")).get()).isEqualTo("Hello You");
  }

  @Test
  void shouldFlatMapOnFailingTry() {
    Try<String> failing = Try.of(failing()).flatMap(value -> Try.of(() -> value + " You"));

    assertThatThrownBy(failing::get).isExactlyInstanceOf(ExecutionException.class).hasCauseExactlyInstanceOf(RuntimeException.class);
  }

  @Test
  void shouldFlatMapWithFailingTry() {
    assertThatThrownBy(Try.of(sucess()).flatMap(value -> Try.of(failing()))::get)
      .isExactlyInstanceOf(ExecutionException.class)
      .hasCauseExactlyInstanceOf(RuntimeException.class);
  }

  @Test
  void shouldNotGetorElseForSuccessfulTry() {
    assertThat(Try.of(sucess()).orElseGet(() -> "Fallback")).isEqualTo("Hello");
  }

  @Test
  void shouldNotGetFaillingOrElseGet() {
    assertThat(Try.of(sucess()).orElseGet(failing())).isEqualTo("Hello");
  }

  @Test
  void shouldGetOrElseGetOnFaillingTry() {
    assertThat(Try.of(failing()).orElseGet(sucess())).isEqualTo("Hello");
  }

  @Test
  void shouldFailOrElseGetOnFaillingTry() {
    assertThatThrownBy(() -> Try.of(failing()).orElseGet(failing()))
      .isExactlyInstanceOf(ExecutionException.class)
      .hasCauseExactlyInstanceOf(RuntimeException.class);
  }

  private Faillable<String> sucess() {
    return () -> "Hello";
  }

  private Faillable<String> failing() {
    return () -> {
      throw new RuntimeException();
    };
  }

  private static record Result(String value) {}
}
